/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "outlook/AppointmentBuilder.h"
//#include "base/Log.h"



/*
* Method to create a SIF String from a calendar Vobject. 
* It get every property name and value and build the XML structure.
*
* @param appointmentStringItem : the string that will contain the xml structure
* @param vo                    : the calendar object that contain every value
*
*/
void VObjectAppointment2SIF(string &appointmentStringItem, VObject *vo) {

    wchar_t* element = NULL;
    wchar_t tempbuf [DIM_LOCAL_TEMP];

    DATE startdate = NULL;
    DATE enddate   = NULL;
    wchar_t startDateValue [DIM_LOCAL_TEMP];
    wchar_t endDateValue   [DIM_LOCAL_TEMP];

    int i = 0;


    appointmentStringItem = TEXT("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
	appointmentStringItem = appointmentStringItem + TEXT("<appointment>");


    if(vo->containsProperty(TEXT("SUMMARY")) && vo->getProperty(TEXT("SUMMARY"))->getValue()) {
        element = vo->getProperty(TEXT("SUMMARY"))->getValue();
		appointmentStringItem  = appointmentStringItem + TEXT("<Subject>") + element + TEXT("</Subject>");
    }
    if(vo->containsProperty(TEXT("LOCATION")) && vo->getProperty(TEXT("LOCATION"))->getValue()) {
        element = vo->getProperty(TEXT("LOCATION"))->getValue();
		appointmentStringItem  = appointmentStringItem + TEXT("<Location>") + element + TEXT("</Location>");
    }
    if(vo->containsProperty(TEXT("DESCRIPTION")) && vo->getProperty(TEXT("DESCRIPTION"))->getValue()) {
        element = vo->getProperty(TEXT("DESCRIPTION"))->getValue();
		appointmentStringItem  = appointmentStringItem + TEXT("<Body>") + element + TEXT("</Body>");
    }
	


	////////////////////////////////////////////////////
	// DTSTART, DTEND:
    // see the start and end date. If the start is 00:00 and end is 23:59 the appointment is decided to be 
    // an all day event. So the AllDayEvent tag is put to 'true' only in the SIF structure.
	////////////////////////////////////////////////////
    // Check startdate, enddate (double format)
	if(vo->containsProperty(TEXT("DTSTART")) && vo->getProperty(TEXT("DTSTART"))->getValue()){
		element = vo->getProperty(TEXT("DTSTART"))->getValue();	
		systemTimeToDouble(element, &startdate, NULL);
	}

	if(vo->containsProperty(TEXT("DTEND")) && vo->getProperty(TEXT("DTEND"))->getValue()) {
		element = vo->getProperty(TEXT("DTEND"))->getValue();
		systemTimeToDouble(element, &enddate, NULL);
	}

    // Check if is AllDayEvent
	BOOL isAllDay = isAllDayEvent(startdate, enddate);
	if(isAllDay)
		wsprintf(tempbuf, TEXT("1"));
	else
		wsprintf(tempbuf, TEXT("0"));
	appointmentStringItem  = appointmentStringItem + TEXT("<AllDayEvent>") + tempbuf + TEXT("</AllDayEvent>");	


    // Write start and end tags (format yyyy-MM-dd if AllDayEvent)
    if (startdate){
        if (isAllDay){
            doubleToSystemTimeBirthday(startDateValue, startdate);
            appointmentStringItem  = appointmentStringItem + TEXT("<Start>") + startDateValue + TEXT("</Start>");
        }
        else {
            element = vo->getProperty(TEXT("DTSTART"))->getValue();
            appointmentStringItem  = appointmentStringItem + TEXT("<Start>") + element + TEXT("</Start>");
        }
    }
    if (enddate){
        if (isAllDay) {
            doubleToSystemTimeBirthday(endDateValue, enddate);
            appointmentStringItem  = appointmentStringItem + TEXT("<End>") + endDateValue + TEXT("</End>");
        }
        else {
            element = vo->getProperty(TEXT("DTEND"))->getValue();
            appointmentStringItem  = appointmentStringItem + TEXT("<End>") + element + TEXT("</End>");
        }
    }




    if(vo->containsProperty(TEXT("CATEGORIES")) && vo->getProperty(TEXT("CATEGORIES"))->getValue()) {
        element = vo->getProperty(TEXT("CATEGORIES"))->getValue();
		appointmentStringItem  = appointmentStringItem + TEXT("<Categories>") + element + TEXT("</Categories>");
    }

	if(vo->containsProperty(TEXT("CLASS")) && vo->getProperty(TEXT("CLASS"))->getValue()) {
        element = vo->getProperty(TEXT("CLASS"))->getValue();
		if(!wcscmp(element, TEXT("PRIVATE")) || !wcscmp(element, TEXT("CONFIDENTIAL")))
			wsprintf(tempbuf, TEXT("%i"), olPrivate);		// Private = 2
		else
			wsprintf(tempbuf, TEXT("%i"), olNormal);		// Normal  = 0
	}
	else
		wsprintf(tempbuf, TEXT("%i"), olNormal);			// Normal  = 0

	appointmentStringItem  = appointmentStringItem + TEXT("<Sensitivity>") + tempbuf + TEXT("</Sensitivity>");



    /****************** Decomment here when recipients supported ************************
	//ATTENDEE;ROLE=OWNER;STATUS=CONFIRMED:John Smith <jsmith@host1.com>
	//ATTENDEE;ROLE=ATTENDEE;STATUS=CONFIRMED;VALUE=URL;TYPE=VCARD:http://www.xyz.com/~myvcard.vcf
	appointmentStringItem  = appointmentStringItem + TEXT("<Recipients>");
	for(i = 0; i<vo->propertiesCount(); i++) {

		if(!wcscmp(vo->getProperty(i)->getName(), TEXT("ATTENDEE")) &&
			vo->getProperty(i)->getValue() &&
			!vo->getProperty(i)->containsParameter(TEXT("VALUE"))) {

            appointmentStringItem  = appointmentStringItem + TEXT("<Recipient>");

            element = vo->getProperty(i)->getValue();

			wchar_t* attendee = new wchar_t[200];
			wcscpy(attendee, vo->getProperty(i)->getValue());
			wchar_t separator[] = TEXT("<");

			wchar_t* pos = NULL;
			pos = wcsstr(attendee, separator);
			if(pos) {
				wchar_t* name = new wchar_t[100];
				wcscpy(name, TEXT(""));
				if((pos - attendee)-1 > 0) {
					wcsncpy(name, attendee, (pos - attendee)-1);
					name[pos - attendee] = TEXT('\0');
				}
				wchar_t* addr = new wchar_t[100];
				wcscpy(addr, pos+1);
				addr[wcslen(addr) - 1] = TEXT('\0');


				appointmentStringItem  = appointmentStringItem + TEXT("<Address>") + addr + TEXT("</Address>");
				appointmentStringItem  = appointmentStringItem + TEXT("<Name>")    + name + TEXT("</Name>");

				delete [] name;  name = NULL;
				delete [] addr;  addr = NULL;
			}

            appointmentStringItem  = appointmentStringItem + TEXT("</Recipient>");
		}
	}
	appointmentStringItem  = appointmentStringItem + TEXT("</Recipients>");
    ***********************************************************************/




	//alarm
	//The value consists of: Run Time, Snooze Time; Repeat Count, Audio Content
	if(vo->containsProperty(TEXT("AALARM")) && vo->getProperty(TEXT("AALARM"))->getValue()) {
		
		wsprintf(tempbuf, TEXT("%i"), 1);	// TRUE
		appointmentStringItem  = appointmentStringItem + TEXT("<ReminderSet>") + tempbuf + TEXT("</ReminderSet>");
		
		DATE runTime;
		systemTimeToDouble(vo->getProperty(TEXT("AALARM"))->getPropComponent(1), &runTime, NULL);

		long minBeforeEvent = long((startdate - runTime) * 1440);
		wsprintf(tempbuf, TEXT("%i"), minBeforeEvent);

		appointmentStringItem  = appointmentStringItem + TEXT("<ReminderMinutesBeforeStart>") + tempbuf + TEXT("</ReminderMinutesBeforeStart>");
	}


	//recurrence 
	//////////////////////////////////////////////////////////////////////////////////
	if((vo->containsProperty(TEXT("RRULE")) && vo->getProperty(TEXT("RRULE"))->getValue())) {

		wsprintf(tempbuf, TEXT("%i"), 1);	// TRUE
		appointmentStringItem  = appointmentStringItem + TEXT("<IsRecurring>") + tempbuf + TEXT("</IsRecurring>");

		string recurrenceString = string();
		bool ret = convertRruleToSIF(recurrenceString, vo->getProperty(TEXT("RRULE"))->getValue(), startdate);

		if(ret)
			appointmentStringItem = appointmentStringItem + recurrenceString;
	}

	else {
		wsprintf(tempbuf, TEXT("%i"), 0);	// FALSE
		appointmentStringItem  = appointmentStringItem + TEXT("<IsRecurring>") + tempbuf + TEXT("</IsRecurring>");
	}
	////////////////////////////////////////////////////////////////////////////////////


	appointmentStringItem = appointmentStringItem + TEXT("</appointment>");
}




/*
* Method that complete a calendar object retrieving property value by an XML structure.
* It parse the XML structure, get the every property value using its name and set it into the VObject.
* Note:
* The returned VObject 'vo' is dinamically allocated inside this method, must be freed by the caller.
*
* @param ptrData  : the variable that contain the xml structure
* @return vo      : the calendar object that contain every value
*/
VObject* SIF2VObjectAppointment(wchar_t *ptrData) {

	VObject* vo = new VObject();

	wchar_t* element   = NULL;
	wchar_t* dateValue = NULL;
    DATE startdate     = NULL;
    DATE enddate       = NULL;
	
	
	vo->addProperty(new VProperty(TEXT("BEGIN"), TEXT("VCALENDAR")));
	vo->addProperty(new VProperty(TEXT("BEGIN"), TEXT("VEVENT")));
	
	element = getElementContent(ptrData, TEXT ("Subject"), NULL);
	if(wcscmp(element, TEXT("")))
		vo->addProperty(new VProperty(TEXT("SUMMARY"), element));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("Location"), NULL);
	if(wcscmp(element, TEXT("")))
		vo->addProperty(new VProperty(TEXT("LOCATION"), element));
	delete [] element; element = NULL;
 
	element = getElementContent(ptrData, TEXT ("Body"), NULL);
	if(wcscmp(element, TEXT("")))
		vo->addProperty(new VProperty(TEXT("DESCRIPTION"), element));
	delete [] element; element = NULL;
 	


    // DTSTART, DTEND: server needs "yyyyMMdd" format if AllDayEvent 
    // ---------------
	element = getElementContent(ptrData, TEXT ("Start"), NULL); 

    if (isDateForAllDayEventFormat(element)){
        systemTimeToDoubleBirthday(element, &startdate);         
        doubleToSystemTimeOnlyDate(element, startdate);       // All day event: from "yyyy-MM-dd" to "yyyyMMdd"
    }
    else
        systemTimeToDouble(element, &startdate, NULL);

	vo->addProperty(new VProperty(TEXT("DTSTART"), element));
	delete [] element; element = NULL;
 

	element = getElementContent(ptrData, TEXT ("End"), NULL);
    if (isDateForAllDayEventFormat(element)){
        systemTimeToDoubleBirthday(element, &enddate);           
        doubleToSystemTimeOnlyDate(element, enddate);         // All day event: from "yyyy-MM-dd" to "yyyyMMdd"
    }
    else
        systemTimeToDouble(element, &enddate, NULL);

	vo->addProperty(new VProperty(TEXT("DTEND"), element));
	delete [] element; element = NULL;
 	

	element = getElementContent(ptrData, TEXT ("Categories"), NULL);
	if(wcscmp(element, TEXT("")))
		vo->addProperty(new VProperty(TEXT("CATEGORIES"), element));
	delete [] element; element = NULL;
 

	long sensitivity;
	element = getElementContent(ptrData, TEXT ("Sensitivity"), NULL);
	sensitivity = _wtoi(element);
	delete [] element; element = NULL;
 
	if(sensitivity == olPrivate)
		vo->addProperty(new VProperty(TEXT("CLASS"), TEXT("PRIVATE")));
	else
		vo->addProperty(new VProperty(TEXT("CLASS"), TEXT("PUBLIC")));


	// Recipients
	///////////////////////////////////////////////////////////////////////
	/********* To be developed *************

	IRecipients* pRecipients;
	pAppointment->get_Recipients(&pRecipients);
	int countRecipient = 0;
	pRecipients->get_Count(&countRecipient);

	for(int i = 0; i < countRecipient; i++) {
		IRecipient* pRecipient;
		pRecipients->Item(i + 1, &pRecipient);

		wchar_t* attedee = new wchar_t[100];
		pRecipient->get_Name(&element);
		element = getElementContent(ptrData, TEXT ("Subject"), NULL);
		wcscpy(attedee, element);
		wcscpy(attedee, TEXT(" "));
		pRecipient->get_Address(&element);
		element = getElementContent(ptrData, TEXT ("Subject"), NULL);
		wcscpy(attedee, element);

		vo->addProperty(new VProperty(TEXT("ATTENDEE"), attedee));

		delete [] attedee; attedee = NULL;
		pRecipient->Release();
	}

	pRecipients->Release();
	*/
	///////////////////////////////////////////////////////////////////////



	BOOL bReminder;
	element = getElementContent(ptrData, TEXT ("ReminderSet"), NULL);
	bReminder = _wtoi(element);
	delete [] element; element = NULL;
 
	if(bReminder == TRUE) {
		long minBefore;
		element = getElementContent(ptrData, TEXT ("ReminderMinutesBeforeStart"), NULL);
		minBefore = _wtoi(element);
		double minStartDate = startdate * 1440;
		//subtract the alarm 
		minStartDate -= minBefore;

		dateValue = new wchar_t[30];
		doubleToSystemTime(dateValue, minStartDate/1440);
		
		wcscat(dateValue, TEXT(";;0;reminder"));
		vo->addProperty(new VProperty(TEXT("AALARM"), dateValue));
		delete [] dateValue; dateValue = NULL;
		delete [] element; element = NULL;
	}


	//recurrence
	///////////////////////////////////////////////////////////////////////
	BOOL isRec;
	element = getElementContent(ptrData, TEXT ("IsRecurring"), NULL);
	isRec = _wtoi(element);
	delete [] element; element = NULL;

	if(isRec) {
		wchar_t* recRule = NULL;

		recRule = convertSIFToRrule(ptrData);

		if(recRule)
			vo->addProperty(new VProperty(TEXT("RRULE"), recRule));
		
		delete [] recRule; recRule = NULL;
	}	
	///////////////////////////////////////////////////////////////////////


	vo->addProperty(new VProperty(TEXT("END"), TEXT("VEVENT")));
	vo->addProperty(new VProperty(TEXT("END"), TEXT("VCALENDAR")));
	
    return vo;
}



/*
* The date in all day event format is yyyy-MM-dd
* To discover if it is in this format find the '-' char inside the string
*/
BOOL isDateForAllDayEventFormat(wchar_t* stringDate) {
        
    BOOL ret = FALSE;
    
    wchar_t* found = NULL;
    found = wcschr(stringDate, '-');
    if (found != NULL) {
        ret = TRUE;
    }

    return ret;

}


BOOL isAllDayEvent(DATE startdate, DATE enddate) {
    
    SYSTEMTIME ststart, stend;
    VariantTimeToSystemTime(startdate, &ststart);
    VariantTimeToSystemTime(enddate, &stend);
    BOOL ret = FALSE;

    if (ststart.wHour == 0 && ststart.wMinute == 0 &&
        stend.wHour == 23 && stend.wMinute == 59) 
        
        ret = TRUE;

    return ret;

}


