/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef INCL_WINDOWS_SYNC_SOURCE
#define INCL_WINDOWS_SYNC_SOURCE

#include "base/fscapi.h"
#include "spds/constants.h"
#include "spds/SyncItem.h"
#include "spds/SyncMap.h"
#include "spds/SyncStatus.h"
#include "spds/SyncSource.h"
#include "outlook/Container.h"


// CALLBACK function pointers.
typedef void  (__stdcall *CBFunction1)(void);					// For get All/Mod/New items	(write to FS, no return value)
typedef BSTR  (__stdcall *CBFunction3)(void);					// For get Del/AllKey items		(return xml)
typedef BSTR  (__stdcall *CBFunction2)(BSTR xmlItem);			// For add items				(pass xml, return LUID)
typedef DWORD (__stdcall *CBFunction4)(BSTR xmlItem);			// For update/delete items		(pass xml, return #items = 1)


// Public function to menage callback adresses.
DWORD WINAPI setCBFunctionsAddresses(long AddressOfgetAllItemsCB,    long AddressOfgetAllItemsContactsCB, 
									 long AddressOfgetNewItemsCB,    long AddressOfgetNewItemsContactsCB,
                                     long AddressOfgetModItemsCB,    long AddressOfgetModItemsContactsCB,
									 long AddressOfgetDelItemsCB,    long AddressOfgetDelItemsContactsCB,
									 long AddressOfgetAllItemsKeyCB, long AddressOfgetAllItemsKeyContactsCB,
                                     long AddressOfaddItemCB,        long AddressOfaddItemContactsCB,
									 long AddressOfupdateItemCB,     long AddressOfupdateItemContactsCB,
                                     long AddressOfdeleteItemCB);



class WindowsSyncSource : public SyncSource {
   
private:

    // The path where the application is located.
    wchar_t* path;

    // The pointer to the returned item (getFirstItem, getNextItem,...)     <- ** Removed (unused, API clones the item passed)
    //SyncItem* syncItem;

    // The container object that contain all, new, update and deleted items
    Container* c;

	// The type of syncSource, can be:
	//	10: contacts
	//	 9: appointments
	//	13: tasks
	//	12: notes
	wchar_t* type;


public:

    /**
     * Constructor: create a SyncSource with the specified name
     *
     * @param name - the name of the SyncSource
	 * @param t    - the type of the SyncSource
     */
    WindowsSyncSource(const wchar_t* name, wchar_t* t) EXTRA_SECTION_01;
	WindowsSyncSource(const wchar_t* name)			   EXTRA_SECTION_01;

    ~WindowsSyncSource() EXTRA_SECTION_01;
    

    void setPath(const wchar_t* p);

    wchar_t* getPath();

	void setType(wchar_t* t);

	wchar_t* getType();

    int beginSync() EXTRA_SECTION_01;
    
    int endSync() EXTRA_SECTION_01;

    void WindowsSyncSource::setItemStatus(const wchar_t* key, int status) EXTRA_SECTION_01;

    void assign(WindowsSyncSource& s) EXTRA_SECTION_01;

    ArrayElement* clone() EXTRA_SECTION_01;


	/* ===========                                             =============
	 * =========== Methods to get single items from the Client =============
	 * ===========                                             =============
	 */
	/*
	* Return the first SyncItem of all. It is used in case of slow or refresh sync 
	* and retrieve the entire data source content.
	* 
	* How it works:
	* -------------
	* - Retrieve ALL items from the client using a CALLBACK function (writes to fs)
	* - Reads ALL Items (from fs) and fills the SyncItem Array
	* - Returns the first Item of all
	*/
    SyncItem* getFirstItem() EXTRA_SECTION_01;

	/*
	* Return the next SyncItem of all. It is used in case of slow or refresh sync 
	* and retrieve the entire data source content.
	*/
    SyncItem* getNextItem() EXTRA_SECTION_01;

	/*
	* Return the first SyncItem of new. It is used in case of fast sync.
	* 
	* How it works:
	* -------------
	* - Retrieve ALL new items from the client using a CALLBACK function (writes to fs)
	* - Reads ALL new Items (from fs) and fills the SyncItem Array
	* - Returns the first Item of new
	*/
    SyncItem* getFirstNewItem() EXTRA_SECTION_01;

	/*
	* Return the next SyncItem of new. It is used in case of fast sync.
	*/
    SyncItem* getNextNewItem() EXTRA_SECTION_01;

	/*
	* Return the first SyncItem of updated. It is used in case of fast sync.
	* 
	* How it works:
	* -------------
	* - Retrieve ALL updated items from the client using a CALLBACK function (writes to fs)
	* - Reads ALL updated Items (from fs) and fills the SyncItem Array
	* - Returns the first Item of updated
	*/
    SyncItem* getFirstUpdatedItem()  EXTRA_SECTION_01;

	/*
	* Return the next SyncItem of updated. It is used in case of fast sync.
	*/
    SyncItem* getNextUpdatedItem() EXTRA_SECTION_01;

	/*
	* Return the first SyncItem of deleted. It is used in case of fast sync.
	* 
	* How it works:
	* -------------
	* - Retrieve ALL deleted items from the client using a CALLBACK function (writes to fs)
	* - Reads ALL deleted Items (from fs) and fills the SyncItem Array
	* - Returns the first Item of deleted
	*/
    SyncItem* getFirstDeletedItem() EXTRA_SECTION_01;

	/*
	* Return the next SyncItem of deleted. It is used in case of fast sync.
	*/
    SyncItem* getNextDeletedItem() EXTRA_SECTION_01;
    
	/*
	* Return the key of the first Item of All, 
	* for refresh-from-server sync.
	*/
    SyncItem* getFirstItemKey();
   
	/*
	* Return the key of the next Item of All, 
	* for refresh-from-server sync.
	*/  
    SyncItem* getNextItemKey();

	/* ===========                                           =============
	 * =========== Methods to set single items on the Client =============
	 * ===========                                           =============
	 */
	/*
	 * Add the item from the server to the client.
	 * Sets the SyncItem key
	 */
    int addItem(SyncItem& item) EXTRA_SECTION_01;
	 
	/*
	 * Update the item from the server to the client.
	 */
    int updateItem(SyncItem& item) EXTRA_SECTION_01;

	/*
	 * Delete the item from the server to the client.
	 */
    int deleteItem(SyncItem& item) EXTRA_SECTION_01;
};

#endif
